#include <config.h>

#if defined(CONFIG_SPI) && (CONFIG_COMMANDS & CFG_CMD_EEPROM)

/*#define DEBUG	1*/
#include <common.h>
#include <asm/io.h>
#include <asm/cacheops.h>
#include <spi.h>
extern spi_chipsel_type spi_chipsel[];

#define SPI_EEPROM_WREN		0x06
#define SPI_EEPROM_RDSR		0x05
#define SPI_EEPROM_READ		0x03
#define SPI_EEPROM_WRITE	0x02
#define EEPROM_PAGE_SIZE	16

#define MAX_TRANCEFER_SIZE 520


/* called from cmd_eeprom.c */
void spi_init_f (void)
{
	debug ( "%s\n", __FUNCTION__ );
}

/* called from cmd_eeprom.c */
ssize_t spi_read (uchar *addr, int alen, uchar *buffer, int len )
{
	char buf[MAX_TRANCEFER_SIZE];
	int start = 0;
	int size;
	ssize_t r = len;

	debug ( "\n%s(%p, %d, %p, %d)\n", __FUNCTION__, addr, alen, buffer, len );
	while ( alen-- ) {
	  start = (start << 8) | *(addr++);
	}
	while ( len > 0 ) {
	  memset( buf, 0xFF, sizeof(buf) );
	  buf[0] = SPI_EEPROM_READ;
	  if ( start >= 0x100 ) buf[0] |= 0x08;
	  buf[1] = start & 0xFF;
	  size = len + 2;
	  if ( size > MAX_TRANCEFER_SIZE )
	    size = MAX_TRANCEFER_SIZE;

	  spi_xfer(spi_chipsel[CFG_SPI_EEPROM_DEVID], size*8, buf, buf);
	  size -= 2;
	  memcpy( buffer, & buf[2], size );
	  len -= size;
	  buffer += size;
	}

	(*spi_chipsel[CFG_SPI_RTC_DEVID])(0);
	return r;
}

/* called from cmd_eeprom.c */
ssize_t spi_write (uchar *addr, int alen, uchar *buffer, int len )
{
	char buf[EEPROM_PAGE_SIZE+2];
	int start = 0;
	int size;
	ssize_t r= len;

	debug ( "\n%s(%p, %d, %p, %d)\n", __FUNCTION__, addr, alen, buffer, len );
	while ( alen-- ) {
	  start = (start << 8) | *(addr++);
	}

	while ( len > 0 ) {
	  /* write protect off */
	  buf[0] = SPI_EEPROM_WREN;
	  spi_xfer(spi_chipsel[CFG_SPI_EEPROM_DEVID], 1*8, buf, buf);

	  buf[0] = SPI_EEPROM_WRITE;
	  if ( start >= 0x100 ) buf[0] |= 0x08;
	  buf[1] = start & 0xFF;

	  size = len;
	  if ( size > EEPROM_PAGE_SIZE )
	    size = EEPROM_PAGE_SIZE;

	  if ( ((start) & ~(EEPROM_PAGE_SIZE-1)) != 
	       ((start + size) & ~(EEPROM_PAGE_SIZE-1)) ) {
	    size = ((start + size) & ~(EEPROM_PAGE_SIZE-1)) - start;
	  }
	  memcpy( &buf[2], buffer, size );
	  spi_xfer(spi_chipsel[CFG_SPI_EEPROM_DEVID], (size+2)*8, buf, buf);
	  len -= size;
	  buffer += size;
	  start += size;

	  do {
	    buf[0] = SPI_EEPROM_RDSR;
	    buf[1] = 0;
	    spi_xfer(spi_chipsel[CFG_SPI_EEPROM_DEVID], 2*8, buf, buf);
	  } while ( buf[1] & 1 )
	      ;
	}
	(*spi_chipsel[CFG_SPI_RTC_DEVID])(0);
	return r;
}

#endif /* defined(CONFIG_SPI) && (CONFIG_COMMANDS & CFG_CMD_EEPROM) */
