/*
 * Board initialize code for TANBAC Evaluation board TB0322.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 */

/*#define DEBUG	1*/
#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <pci.h>

#define GIUIOSELL *((volatile unsigned short*)0xAF003140)
#define GIUPIODL  *((volatile unsigned short*)0xAF003144)
#define GPIO15_MASK 0x8000

#if defined(CONFIG_PCI)
static struct pci_controller hose;

extern void init_vr4155_pci(struct pci_controller*);

void pci_init_board (void)
{
	init_vr4155_pci(&hose);
}
#endif

long int initdram(int board_type)
{
	return get_ram_size ((volatile long *)CFG_SDRAM_BASE, 0x8000000);
}


#if defined(CONFIG_SPI)
#include <spi.h>

/* GPIO15  H: select RTC
 *         L: select EEPROM
 */
void spi_eeprom_chipsel(int cs)
{
	debug ( "%s(%d)\n", __FUNCTION__, cs );

	if ( cs ) {
	  /* RTC's CE must be less than 1S. so always eeprom's CE is
	     active */
	  GIUPIODL |= GPIO15_MASK;	/* deactivate the chip select */
	  udelay( 60 );
	}
	GIUPIODL &= ~GPIO15_MASK;	/* activate the chip select */
}

void spi_rtc_chipsel(int cs)
{
	debug ( "%s(%d)\n", __FUNCTION__, cs );
	if(cs)
	  GIUPIODL |= GPIO15_MASK;	/* activate the chip select */
	else
	  GIUPIODL &= ~GPIO15_MASK;
}

/*
 * The SPI command uses this table of functions for controlling the SPI
 * chip selects: it calls the appropriate function to control the SPI
 * chip selects.
 */
spi_chipsel_type spi_chipsel[] = {
	spi_eeprom_chipsel,
	spi_rtc_chipsel
};
int spi_chipsel_cnt = sizeof(spi_chipsel) / sizeof(spi_chipsel[0]);

#endif /* CONFIG_SPI */

#ifdef CONFIG_MISC_INIT_R
#define CMU_CLK_MSK	0xAF000060 /* clock mask */
#define CMU_CLK_MSK_CSI	(1<<5)

#define SCUPINMUX3REG	0xAF008008
#define SCUPINMUX3_CSI_MASK	(0x00FC0000)
#define SCUPINMUX3_CSI	  	(0x00540000)
#define SCUPINMUX3_NMIB_MASK	(0x00030000)
#define SCUPINMUX3_NMIN_GPIO23	(0x00010000)

#define SPI_EEPROM_READ		0x03
/* ------------------------------------------------------------------------- */
int misc_init_r(void)
{
#if defined(CONFIG_SPI)
	int i;
	uchar eeprom[16];
	DECLARE_GLOBAL_DATA_PTR;

	debug ( "%s(%d)\n", __FUNCTION__ );
	GIUPIODL &= ~GPIO15_MASK;
	GIUIOSELL |= GPIO15_MASK; /* CSI chipsel */

#ifdef CONFIG_SOFT_SPI
	*((volatile unsigned int *)(CMU_CLK_MSK)) &= ~CMU_CLK_MSK_CSI;
	*((volatile unsigned int *)(SCUPINMUX3REG)) &= ~SCUPINMUX3_CSI_MASK;
	*((volatile unsigned int *)(SCUPINMUX3REG)) |= SCUPINMUX3_CSI;
#endif

	/* relocate function pointer */
	for ( i = 0; i < sizeof(spi_chipsel) / sizeof(spi_chipsel[0]); i++ )
	  spi_chipsel[i] = (spi_chipsel_type)(((uchar *)spi_chipsel[i]) + gd->reloc_off);

	spi_init();
	eeprom[0] = SPI_EEPROM_READ;
	eeprom[1] = 0;
	spi_xfer(spi_chipsel[CFG_SPI_EEPROM_DEVID], sizeof(eeprom)*8, eeprom, eeprom);
	memcpy (gd->bd->bi_enetaddr, &eeprom[2], 6);
#ifdef CONFIG_HAS_ETH1
	memcpy (gd->bd->bi_enet1addr, &eeprom[10], 6);
#endif
#endif /* CONFIG_SPI */

	/* Power Button set to GPIO23 */
	*((volatile unsigned int *)(SCUPINMUX3REG)) &= ~SCUPINMUX3_NMIB_MASK;
	*((volatile unsigned int *)(SCUPINMUX3REG)) |= SCUPINMUX3_NMIN_GPIO23;

	return 0;
}
#endif /* CONFIG_MISC_INIT_R */

int checkboard (void)
{
	printf("Board: TANBAC TB0322 ");
	printf("(CPU Speed %d MHz)\n", (int)CPU_CLOCK/1000000);

	return 0;
}
