/* NEC Vr4155 Gigabit Ethernet driver
 *
 * (C) Copyright 2007 Media Lab. <linux@mlb.co.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

#include <config.h>

#ifdef CONFIG_VR4155_ETH

#include <common.h>
#include <malloc.h>
#include <net.h>
#include <miiphy.h>
#include <command.h>
#include <asm/io.h>
#include <asm/cacheops.h>

#define ETH0_BASE	0xAF010000
#define ETH1_BASE	0xAF011000
#define DEVNAME		"bobcat"

#define MACC1		0x0000
  #define MACC1_TEN	0x0001
  #define MACC1_REN	0x0004
#define MACC2		0x0004
#define   MACC2_PRLEN 0x7000
#define   MACC2_IFMOD_1000 0x0200
#define   MACC2_IFMOD_100 0x0100
#define   MACC2_HUGEN 0x0020
#define   MACC2_PADEN 0x0004
#define   MACC2_CRCEN 0x0002
#define   MACC2_FULLD 0x0001

#define IPGIFG		0x0008
#define HDREG		0x000c
#define LMAX		0x0010
#define MIIC		0x0020
  #define MIIC_SRST	0x80000000
#define MCMD		0x0024
  #define MCMD_RSTAT	0x0001
#define MADR		0x0028
#define MWTD		0x002c
#define MSTA		0x0030
#define MIND		0x0034
  #define MIND_BUSY	0x0001
  #define MIND_NVAL	0x0004

#define LSA1		0x0040
#define LSA2		0x0044
#define AFR		0x0084
  #define AFR_PRO	0x0010
  #define AFR_AUC	0x0008
  #define AFR_PRM	0x0004
  #define AFR_ABC	0x0001
#define TIMR		0x009c
#define RIMR		0x00a0
#define TSVREG		0x00a4
#define RSVREG		0x00a8

#define RBYT		0x021c
#define RPKT		0x0220

#define XMITCFG		0x0400
  #define XMITCFG_TEN	0x80000000
  #define XMITCFG_TXSHPR 0x00800000  // SW queue first
  #define XMITCFG_DCM	0x00080000   // Descripter Clear Mode
#define XMITCTRL	0x0404
#define XMITSTAT	0x0408
#define XMITDTRR	0x040c
#define XMITDPRA	0x0410
#define XMITDPRB	0x0414
#define XMITDPRC	0x0418
#define XMITDPRD	0x041c
#define XMITMODE1	0x0420
#define XMITMODE2	0x0424
#define XMITDIS		0x0428
#define RCVCFG		0x0440
#define RCVCTRL		0x0444
#define RCVSTAT		0x0448
#define RCVDPRA		0x044c
#define RCVDPRC		0x0454
#define RCVPDRA		0x045c
#define RCVPDRC		0x0464
#define RCVMODE1	0x046c
#define RCVMODE2	0x0470
#define RCVERCNT	0x0478
#define RCVBFCNT	0x047c
#define RCVDMSK		0x0488
#define RCVDIS		0x048c
#define FLWCNT		0x04b0
#define ISR1		0x04c8
#define ISR1MSK		0x04cc
#define ISR2		0x04d0
#define ISR2MSK		0x04d4
#define XMITSHA		0x04e8

#define ECR		0x0500
#define PCR1		(ETH0_BASE + 0x0504)
  #define PCR1_PLLLOCK	0x0200

#define CLASCNT		0x0700
#define CLASRCNT		0x0710
#define HWRCNT		0x0800
#define PKTCHK		0x0900

#define ETH0_PHY_ADDR	1
#define ETH1_PHY_ADDR	1

/*---------------------------------------------------------------------------*/
/* MII register offsets                                                      */
/*---------------------------------------------------------------------------*/
#define     MII_CONTROL         0x0000
#define     MII_STATUS          0x0001
#define     MII_PHYID1          0x0002
#define     MII_PHYID2          0x0003
#define     MII_ANAR            0x0004
#define     MII_ANLPAR          0x0005
#define     MII_ANER            0x0006
#define     MII_GBSR            0x000a
#define     MII_LBREMR          0x0018
#define     MII_PAR             0x0019
/*---------------------------------------------------------------------------*/
/* MII Control register bit definitions.                                     */
/*---------------------------------------------------------------------------*/
#define     MIICNTL_SPEED1      0x0040
#define     MIICNTL_FDX         0x0100
#define     MIICNTL_RST_AUTO    0x0200
#define     MIICNTL_ISOLATE     0x0400
#define     MIICNTL_PWRDWN      0x0800
#define     MIICNTL_AUTO        0x1000
#define     MIICNTL_SPEED       0x2000
#define     MIICNTL_LPBK        0x4000
#define     MIICNTL_RESET       0x8000
/*---------------------------------------------------------------------------*/
/* MII Status register bit significance.                                     */
/*---------------------------------------------------------------------------*/
#define     MIISTAT_EXT         0x0001
#define     MIISTAT_JAB         0x0002
#define     MIISTAT_LINK        0x0004
#define     MIISTAT_CAN_AUTO    0x0008
#define     MIISTAT_FAULT       0x0010
#define     MIISTAT_AUTO_DONE   0x0020
#define     MIISTAT_CAN_T       0x0800
#define     MIISTAT_CAN_T_FDX   0x1000
#define     MIISTAT_CAN_TX      0x2000
#define     MIISTAT_CAN_TX_FDX  0x4000
#define     MIISTAT_CAN_T4      0x8000
/*---------------------------------------------------------------------------*/
/* MII GBSR register Bits                                                    */
/*---------------------------------------------------------------------------*/
#define     MII_NWAY_1000      0x0400
#define     MII_NWAY_1000_FDX  0x0800
/*---------------------------------------------------------------------------*/
/* MII ANAR and ANLPAR register Bits                                         */
/*---------------------------------------------------------------------------*/
#define     MII_NWAY_NODE_SEL   0x001f
#define     MII_NWAY_CSMA_CD    0x0001
#define     MII_NWAY_T          0x0020
#define     MII_NWAY_T_FDX      0x0040
#define     MII_NWAY_TX         0x0080
#define     MII_NWAY_TX_FDX     0x0100
#define     MII_NWAY_T4         0x0200
#define     MII_NWAY_RF         0x2000
#define     MII_NWAY_ACK        0x4000
#define     MII_NWAY_NP         0x8000


/* descriptor count */
#define NUM_TX_DESC PKTBUFSRX
#define NUM_RX_DESC 16   // should be 16 for Ring buffer

/* descriptor pointer */
static int next_rx = 0;
static int now_rx = 0;
static int now_tx = 0;

#define NEXTTXDESC(n) ((n)<NUM_TX_DESC-1?(n)+1:0)
#define NEXTRXDESC(n) ((n)<NUM_RX_DESC-1?(n)+1:0)

typedef struct{
	u16 size;
	u16 flag;
	u32 addr;
} DESC;

typedef struct{
	u16 size;
	u16 flag;
	u32 addr;
	u32 level;
	u32 pad;
} RDESC;

static DESC tx_descriptor[NUM_TX_DESC+1]__attribute__((aligned(16)));
static RDESC rx_descriptor[NUM_RX_DESC+1]__attribute__((aligned(16)));

#define RXBUFFSIZE 2048
static char rx_buff[RXBUFFSIZE*(NUM_RX_DESC+1)]__attribute__((aligned(16)));

struct vr4155_enet_hw {
	void *hw_addr;
	uint32_t phy_addr;
	int media;
};

#define MAX_WAIT 1000

/* media type selection */
#define MEDIA_NONE   	0
#define MEDIA_10_HALF   1
#define MEDIA_10_FULL   2
#define MEDIA_100_HALF  3
#define MEDIA_100_FULL  4
#define MEDIA_1000_HALF 5
#define MEDIA_1000_FULL 6

// bits in descripter
#define LAST  0x8000
#define DB_LP 0x4000
#define OWN   0x2000

static int ether_adr_init(struct eth_device* dev);
static void ether_hw_reset(struct vr4155_enet_hw *hw);
static int ether_int_init(struct vr4155_enet_hw *hw);
static int ether_mac_init(struct vr4155_enet_hw *hw);
static void ethdsc_init(struct vr4155_enet_hw *hw);
static int mdio_write(struct vr4155_enet_hw* hw, int phyreg_no, int data);
static unsigned int mdio_read(struct vr4155_enet_hw* hw, int phyreg_no);
static void ether_link_status(struct vr4155_enet_hw* hw, int reg, int reg2);
static int ether_auto_negotiation(struct eth_device* dev);
extern volatile uchar	PktBuf[(PKTBUFSRX+1) * PKTSIZE_ALIGN + PKTALIGN];

/* L1 D-cache writeback macro for VR4155 4 way cache */
static inline void L1_D_writeback(ulong addr)
{
  __asm__ __volatile__ (
    ".set noreorder\n\t"
    "cache %1,  (%0)\n\t"
    "cache %1, 0x2000(%0)\n\t"
    "cache %1, 0x4000(%0)\n\t"
    "cache %1, 0x6000(%0)\n\t"
    " .set reorder"
    :
    : "r" (addr),
      "i" (Hit_Writeback_D)
  );
}

void static inline y_sync(void)
{
        __asm__ volatile ("sync");
}

/**************************************************************************/
/* SEND - Transmit a frame                                                */
/***************************************************************************/
static int vr4155_enet_send(struct eth_device* dev, volatile void *packet, int length){
  volatile DESC *tx_desc = (volatile DESC*)(KSEG1ADDR(&tx_descriptor[0]));
	struct vr4155_enet_hw* hw = (struct vr4155_enet_hw*)dev->priv;
	volatile u32 *tx_dtrr = (volatile u32*)(hw->hw_addr+XMITDTRR);
	//volatile u32 *tx_status = (volatile u32*)(hw->hw_addr+XMITSTAT);

	int i, line;
	int res;
	u32 buff_addr;
	int next_tx;

	//printf("vr4155_enet_send:%x:%x\n",packet,length);

	/* tx data buffer must be writeback */
	/* 16byte per cache line for vr4155 */
	buff_addr = (u32)packet;
	line = (length+0x10 -1) / 0x10;
	for(i=0; i<=line; i++){
	         L1_D_writeback(buff_addr);
		 buff_addr += 0x10;
	}
	
	/* prepair tx descriptor */
	tx_desc[now_tx].size = length;
	tx_desc[now_tx].flag =  LAST | DB_LP;
	tx_desc[now_tx].addr = (virt_to_phys(packet));
	next_tx = NEXTTXDESC(now_tx);
	tx_desc[next_tx].size = 0;
	tx_desc[next_tx].flag = DB_LP | OWN;
	tx_desc[next_tx].addr = 0;

	//printf("desc:%x:%x:%x:%x\n",now_tx, tx_desc[now_tx].size, tx_desc[now_tx].flag, tx_desc[now_tx].addr);
	
	/* tx DMA enable */
	*tx_dtrr = 0x0001;        // DMA request
	
#define TX_MAX_WAIT 100
	/* wait Tx DMA Done */
	udelay(1);
	i=0;
	while(!(tx_desc[now_tx].flag & OWN)){
		if(i>TX_MAX_WAIT){
			printf("TX timeout:%x \n", (tx_desc[now_tx].flag));
			break;
		}
		udelay(10);
		i++;
	}
	//printf("txstat:%x:%x:%x\n",*tx_status,i,*tx_dtrr);

	/* Clear done bit */
	tx_desc[now_tx].size = 0;
	tx_desc[now_tx].flag = 0;
	tx_desc[now_tx].addr = 0;

	res = tx_desc[now_tx].flag;
	now_tx = next_tx;
	return(res);
}

/**************************************************************************/
/* RECV - Receive a frame                                                 */
/**************************************************************************/
/***** data output from now rx-descriptor to data buffer *****/
/* return value
 *  0: receive is OK
 *  1: nothing has been received
 *  2: receive data is ERROR
 */
static int vr4155_enet_recv(struct eth_device* dev)
{
  volatile RDESC *rx_desc =(volatile RDESC*)(KSEG1ADDR(&rx_descriptor[0]));
	//volatile u32 *rxcfgr = (volatile u32*)(hw->hw_addr+RCVCFG);
	//volatile u32 *rx_stat = (volatile u32*)(hw->hw_addr+RCVSTAT);
	//volatile u32 *rxdpr = (volatile u32*)(hw->hw_addr+RCVDPRA);
	//volatile u32 *isr1 = (volatile u32*)(hw->hw_addr+ISR1);
	//volatile u32 *isr2 = (volatile u32*)(hw->hw_addr+ISR2);

	int idx;
	int length;
	u16 status;

#define RXOK 0x0001
	idx = now_rx;

	/* nothing has been received */
	if(!(rx_desc[idx].flag & OWN)) return 1;

	//printf("rcv %x:%x:%x\n",now_rx,*rx_stat,*rxdpr);
	//printf("isr:%x:%x\n",*isr1,*isr2);
	/* receive OK */
	status = rx_desc[idx].flag;
	length = rx_desc[idx].size & 0x3fff;
	//printf("length %x:%x\n",length,status);
	if( !(status & RXOK)) {
	  printf("[Rx Error] status:%04x \n", status);
	  return 2;
	}
	else {
	  /* Pass the packet up to the protocol layyers */
	  NetReceive((void*) KSEG1ADDR(&rx_buff[RXBUFFSIZE*idx]), length);
	}
	
	/* free current descriptor */
	rx_desc[idx].flag = DB_LP;
	rx_desc[idx].size = RXBUFFSIZE;
	now_rx = NEXTRXDESC(idx);
	return 0;
}

/**************************************************************************/
/* INIT - Look for an adapter, this routine's visible to the outside      */
/**************************************************************************/
static int vr4155_enet_init(struct eth_device* dev, bd_t * bd)
{
	struct vr4155_enet_hw* hw = (struct vr4155_enet_hw*)dev->priv;
	int status = 1;

	ether_hw_reset(hw);
	ether_adr_init(dev);

	status = ether_auto_negotiation(dev);

	ethdsc_init(hw);
	ether_mac_init(hw);
	ether_int_init(hw);

	return status;
}

/*====================  vr4155_enet MAC Unit initialize  ===================*/
static int ether_mac_init(struct vr4155_enet_hw* hw)
{
  volatile DESC *tx_desc = (volatile DESC*)(KSEG1ADDR(&tx_descriptor[0]));
  volatile RDESC *rx_desc =(volatile RDESC*)(KSEG1ADDR(&rx_descriptor[0]));
//	volatile u32 *macc2 = (volatile u32*)(hw->hw_addr+MACC2);
//	volatile u32 *ecr = (volatile u32*)(hw->hw_addr+ECR);
	volatile u32 *afr = (volatile u32*)(hw->hw_addr+AFR);
	volatile u32 *xmitcfg = (volatile u32*)(hw->hw_addr+XMITCFG);
	volatile u32 *xmitctrl = (volatile u32*)(hw->hw_addr+XMITCTRL);
	volatile u32 *xmitmode1 = (volatile u32*)(hw->hw_addr+XMITMODE1);
	volatile u32 *xmitmode2 = (volatile u32*)(hw->hw_addr+XMITMODE2);
	volatile u32 *xmitdis = (volatile u32*)(hw->hw_addr+XMITDIS);
	volatile u32 *tx_dpr = (volatile u32*)(hw->hw_addr+XMITDPRA);
	volatile u32 *rxcfgr = (volatile u32*)(hw->hw_addr+RCVCFG);
	volatile u32 *rx_cnt = (volatile u32*)(hw->hw_addr+RCVCTRL);
	volatile u32 *rxdpr = (volatile u32*)(hw->hw_addr+RCVDPRA);
	volatile u32 *rxmode1 = (volatile u32*)(hw->hw_addr+RCVMODE1);
	volatile u32 *rxmode2 = (volatile u32*)(hw->hw_addr+RCVMODE2);
	volatile u32 *rxdmamask = (volatile u32*)(hw->hw_addr+RCVDMSK);
//	volatile u32 *isr1 = (volatile u32*)(hw->hw_addr+ISR1);
//	volatile u32 *isr2 = (volatile u32*)(hw->hw_addr+ISR2);
	volatile u32 *isr1msk = (volatile u32*)(hw->hw_addr+ISR1MSK);
	volatile u32 *isr2msk = (volatile u32*)(hw->hw_addr+ISR2MSK);

	//volatile u32 *rxdis = (volatile u32*)(hw->hw_addr+RCVDIS);
	//volatile u32 *pktchk = (volatile u32*)(hw->hw_addr+PKTCHK);
	//volatile u32 *rxpool = (volatile u32*)(hw->hw_addr+RCVPDRA);

	/*
	if(hw->media ==  MEDIA_1000_FULL){
	  *macc2 =  MACC2_PRLEN | MACC2_IFMOD_1000 | MACC2_PADEN | MACC2_FULLD;
	  *ecr = 0x20080000;  // GMII mode
	} else if(hw->media ==  MEDIA_1000_HALF){
	  *macc2 =  MACC2_PRLEN | MACC2_IFMOD_1000 | MACC2_PADEN;
	  *ecr = 0x20080000;  // GMII mode
	} else if((hw->media ==  MEDIA_100_FULL)||
		  (hw->media ==  MEDIA_10_FULL)){
	  *macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN | MACC2_FULLD;
	  *ecr = 0x00080000;
	} else {
	  *macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN;
	  *ecr = 0x00080000;
	}
	*/

	// transmit DMA settings
	*xmitcfg = 0x00800000;   // SW Queue first, 1 burst (8bytes)
	*xmitctrl = 0x04001f80;  // Drain:128x8bytes Fill:max
	*xmitmode1 = 0x00000001; // Ring buffer
	*xmitmode2 = 0;          // Queue A 16descs
	*xmitdis = 0x0f0e;       // enable Queue A only
	*tx_dpr = (virt_to_phys(tx_desc));
	*xmitcfg = 0x80800000;   // TX on

	// receive address filter
	*afr = AFR_AUC|AFR_ABC; // unicast or broadcast
	//*afr = AFR_PRO;       // Promiscus mode

	// receive DMA settings
	*rxcfgr = 0x00000007;
	*rxmode1 = 0x00000001;  // Ring buffer mode
	*rxmode2 = 0x00000000;  // 16 descs
	*rx_cnt  = 0x00000080;  // Drain threthold: this should be burst size
	*rxdpr   = virt_to_phys(rx_desc);
	*rxcfgr  = 0x80040007;  // Rx on, burst 128 bytes
	*rxdmamask = 0xe0000100;// AHB bus mask for rcv DMA  

	// clear isr,isr1
	debug("clear isr:%x:%x\n",*isr1,*isr2);
	*isr1msk = 0x04;
	*isr2msk = 0x03;

	//*rxdis = 0x00000f0a;    // Queue A only
	//*hwrcnt = 0x00000000;
	//*pktchk = 0x01010100;
	//*rxcfgr = 0x80000007;

	return 0;
}

/*=====================  vr4155_enet Int initialize  ====================*/
static int ether_int_init(struct vr4155_enet_hw *hw){

	// TIMR and RIMR is cleared at reset

	// MISRC is cleared at reset

	// MSR1 MSR2 is cleared at reset
	// EMSR is cleared at reset
	
	return 0;
}

/*====================  set MAC address  =============================*/
static int ether_adr_init(struct eth_device* dev){
	struct vr4155_enet_hw* hw = (struct vr4155_enet_hw*)dev->priv;
	volatile u32 *lsa1 = (volatile u32*)(hw->hw_addr+LSA1);
	volatile u32 *lsa2 = (volatile u32*)(hw->hw_addr+LSA2);
	u32 mac_addr_hi, mac_addr_lo;

        mac_addr_hi  =	(dev->enetaddr[1] << 24) | (dev->enetaddr[0] << 16) ;
        mac_addr_lo  =	(dev->enetaddr[5] << 24) | (dev->enetaddr[4] << 16) |
			(dev->enetaddr[3] <<  8) | (dev->enetaddr[2]      ) ;
	
	/*
	printf( "ether_adr_init: %x:%x:%x:%x:%x:%x\n",
			(mac_addr_hi >> 16) & 0xFF,
			(mac_addr_hi >> 24) & 0xFF,
			(mac_addr_lo )      & 0xFF,
			(mac_addr_lo >> 8)  & 0xFF,
			(mac_addr_lo >> 16) & 0xFF,
			(mac_addr_lo >> 24) & 0xFF);
	*/

	*lsa2 = mac_addr_hi;
	*lsa1 = mac_addr_lo;

	return 0;
}

/*==============  hardware initialization  =====================*/
static void ether_hw_reset(struct vr4155_enet_hw *hw){
	volatile u32 *ecr = (volatile u32*)(hw->hw_addr+ECR);
	volatile u32 *pcr1 = (volatile u32*)(PCR1);
	volatile u32 *miic = (volatile u32*)(hw->hw_addr+MIIC);
	volatile u32 *macc1 = (volatile u32*)(hw->hw_addr+MACC1);
	//volatile u32 *macc2 = (volatile u32*)(hw->hw_addr+MACC2);
	//volatile u32 *ipg = (volatile u32*)(hw->hw_addr+IPGIFG);
	//volatile u32 *hdreg = (volatile u32*)(hw->hw_addr+HDREG);
	//volatile u32 *lmax = (volatile u32*)(hw->hw_addr+LMAX);
	//volatile u32 *fqaddr = (volatile u32*)(0xa1012004);
	//volatile u32 *fqcfg = (volatile u32*)(0xa1012000);

	*ecr = 0x0000ffff;  // all reset
	udelay(1000);
	if ( hw == (void*)ETH0_BASE || !(*pcr1 & PCR1_PLLLOCK) ) {
	    *pcr1 = 0x02;       // start PLL

	    // wait for PLL lock
	    //printf("pcr1:%x\n",*pcr1);
	    while ((*pcr1 & PCR1_PLLLOCK) == 0) {
		udelay(10);
	    }
	}
	//printf("pcr1:%x\n",*pcr1);

	// free queue settings
	//*fqaddr = 0x400000;
	//*fqcfg = 0x80000101;

	// ecr DPS through
	*ecr = 0x00080000;

	// MAC reset 
	*macc1 = 0xc00f0010;
	udelay(1000);
	//*macc1 = 0x00000010;
	*macc1 = 0x00000035;

	/*
	*macc2 = 0x00007225;
	*ipg = 0x40605060;
	*hdreg = 0x0000f3ff;
	*lmax = 0x00006000;
	*/

	*miic = MIIC_SRST | 0x7;  // reset MII
	udelay(100000);
	*miic = 0x7;              // MDC=4.7MHz
	udelay(100000);

}


/**************************************************************************/
/* MDIO treatment                                                         */
/**************************************************************************/
/***** auto negotiation *****/
static int ether_auto_negotiation(struct eth_device* dev)
{
	int reg,reg2,bAutoNego,i;
	struct vr4155_enet_hw* hw = (struct vr4155_enet_hw*)dev->priv;

	//mdio_write(hw, MII_CONTROL, MIICNTL_AUTO);  /* ANEGEN */
	//mdio_write(hw, MII_CONTROL,  MIICNTL_AUTO | MIICNTL_RST_AUTO);  /* restart auto nego */
	//mdio_write(hw, MII_CONTROL, 0x1140);  /* auto nego */

	bAutoNego = 0;

	printf( "%s: Auto Negotiation ", dev->name );
	//	mdio_write(hw,MII_CONTROL,MIICNTL_RESET);  // PHY reset 
	//udelay(200000);

	for(i=0; i < 1000; i++) {
		reg = mdio_read(hw, MII_STATUS);
		if(reg & MIISTAT_AUTO_DONE) {
			bAutoNego = 1;
			break;
		}
		udelay(10000);
	}

	debug("MII_STATUS:%x\n",reg);
	if(!bAutoNego){                         /* Setup 10BASE-T Half duplex */
//		mdio_write(hw, MII_CONTROL, 0x0000);
		printf("failed ");
	} 
	reg = mdio_read(hw, MII_ANLPAR);
	reg2 = mdio_read(hw, MII_GBSR);
	debug("MII_ANLPAR:%x MII_GBSR:%x\n",reg,reg2);
	ether_link_status(hw, reg, reg2);
	return bAutoNego;
}

static void ether_link_status(struct vr4155_enet_hw* hw, int reg, int reg2)
{
	volatile u32 *macc2 = (volatile u32*)(hw->hw_addr+MACC2);
	volatile u32 *ecr = (volatile u32*)(hw->hw_addr+ECR);

 	if(reg2 & MII_NWAY_1000_FDX){
		printf("complete: 1000Base Full duplex\n");
		mdio_write(hw, MII_CONTROL, MIICNTL_AUTO | MIICNTL_SPEED1 | MIICNTL_FDX);
		hw->media = MEDIA_1000_FULL;
		*macc2 =  MACC2_PRLEN | MACC2_IFMOD_1000 | MACC2_PADEN | MACC2_FULLD;
		*ecr = 0x20080000;  // GMII mode
	}
	else if(reg2 & MII_NWAY_1000){
		printf("complete: 1000Base Half duplex\n");
		mdio_write(hw, MII_CONTROL, MIICNTL_AUTO | MIICNTL_SPEED1);
		hw->media = MEDIA_1000_HALF;
		*macc2 =  MACC2_PRLEN | MACC2_IFMOD_1000 | MACC2_PADEN;
		*ecr = 0x20080000;  // GMII mode
	}		
	else if(reg & MII_NWAY_TX_FDX){
		printf("complete: 100Base Full duplex\n");
		mdio_write(hw, MII_CONTROL, MIICNTL_AUTO | MIICNTL_SPEED | MIICNTL_FDX);
		hw->media = MEDIA_100_FULL;
		*macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN | MACC2_FULLD;
		*ecr = 0x00080000;
	}
	else if(reg & MII_NWAY_TX){
		printf("complete: 100Base Half duplex\n");
		mdio_write(hw, MII_CONTROL, MIICNTL_AUTO | MIICNTL_SPEED);
		hw->media = MEDIA_100_HALF;
		*macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN;
		*ecr = 0x00080000;
	}
	else if(reg &  MII_NWAY_T_FDX){
		printf("complete: 10Base Full duplex\n");
		mdio_write(hw, MII_CONTROL, MIICNTL_AUTO | MIICNTL_FDX);
		hw->media = MEDIA_10_FULL;
		*macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN | MACC2_FULLD;
		*ecr = 0x00080000;
	}
	else {
	  printf("complete: 10Base Half duplex\n");
	  mdio_write(hw, MII_CONTROL, MIICNTL_AUTO );
	  hw->media = MEDIA_10_HALF;
	  *macc2 =  MACC2_PRLEN | MACC2_IFMOD_100 | MACC2_PADEN;
	  *ecr = 0x00080000;
	}
}

#define FIAD_SHIFT 8

static unsigned int mdio_read(struct vr4155_enet_hw* hw, int phyreg_no){
	volatile u32 *madr = (volatile u32*)(hw->hw_addr+MADR);
	volatile u32 *mcmd = (volatile u32*)(hw->hw_addr+MCMD);
	volatile u32 *mind = (volatile u32*)(hw->hw_addr+MIND);
	volatile u32 *msta = (volatile u32*)(hw->hw_addr+MSTA);

	/* phy addr & cmd set */
	*madr = (hw->phy_addr <<  FIAD_SHIFT) | phyreg_no;
	*mcmd = MCMD_RSTAT;

	/* wait not busy = read is comlited */
	while( *mind & MIND_BUSY);

	return *msta;
}

static int mdio_write(struct vr4155_enet_hw* hw, int phyreg_no, int data){
	volatile u32 *madr = (volatile u32*)(hw->hw_addr+MADR);
	volatile u32 *mind = (volatile u32*)(hw->hw_addr+MIND);
	volatile u32 *mwtd = (volatile u32*)(hw->hw_addr+MWTD);

	/* phy addr & reg set */
	*madr = (hw->phy_addr <<  FIAD_SHIFT) | phyreg_no;

	/* data write */
	*mwtd = data;

	/* wait not busy = write is comlited */
	while( *mind & MIND_BUSY);

	if ( *mind & MIND_NVAL )
	  return 1;

	return 0;
}

#if defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII)
/**
 * Read a 16-bit value from an MII register.
 */
static int vr4155_miiphy_read(char * devname, unsigned char const addr,
		unsigned char const reg, unsigned short * const value)
{
	struct eth_device *dev;
	struct vr4155_enet_hw* hw;
	volatile u32 *madr;
	volatile u32 *mcmd;
	volatile u32 *mind;
	volatile u32 *msta;


	dev = eth_get_dev_by_name(devname);

	/* Parameter checks */
	if (dev == NULL || strncmp( dev->name, DEVNAME, sizeof(DEVNAME)-1 ) ) {
		return 1;
	}

	hw = (struct vr4155_enet_hw*)dev->priv;

	madr = (volatile u32*)(hw->hw_addr+MADR);
	mcmd = (volatile u32*)(hw->hw_addr+MCMD);
	mind = (volatile u32*)(hw->hw_addr+MIND);
	msta = (volatile u32*)(hw->hw_addr+MSTA);

	/* phy addr & cmd set */
	*madr = (addr <<  FIAD_SHIFT) | reg;
	*mcmd = MCMD_RSTAT;

	/* wait not busy = read is comlited */
	while( *mind & MIND_BUSY);

	*value = *msta;

	if ( *mind & MIND_NVAL )
	  return 1;

	return 0;
}

static int vr4155_miiphy_write(char * devname, unsigned char const addr,
		unsigned char const reg, unsigned short const value)
{
	struct eth_device *dev;
	struct vr4155_enet_hw* hw;

	dev = eth_get_dev_by_name(devname);

	/* Parameter checks */
	if (dev == NULL || strncmp( dev->name, DEVNAME, sizeof(DEVNAME)-1 ) ) {
		return 1;
	}

	hw = (struct vr4155_enet_hw*)dev->priv;

	return mdio_write( hw, reg, value );
}
#endif /* defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII) */


/**************************************************************************/
/* Descriptor treatment                                                   */
/**************************************************************************/
static void ethdsc_init(struct vr4155_enet_hw* hw)
{
  volatile DESC *tx_desc = (volatile DESC*)(KSEG1ADDR(&tx_descriptor[0]));
  volatile RDESC *rx_desc =(volatile RDESC*)(KSEG1ADDR(&rx_descriptor[0]));
	volatile u32 *rx_dpr = (volatile u32*)(hw->hw_addr+RCVDPRA);
	volatile u32 *tx_dpr = (volatile u32*)(hw->hw_addr+XMITDPRA);
	int i;

	debug("tx_desc:%x rx_desc:%x \n", &tx_desc[0], &rx_desc[0]);

	for(i=0; i<NUM_RX_DESC; i++){
		rx_desc[i].size = RXBUFFSIZE;
		//rx_desc[i].size = PKTSIZE_ALIGN;
		rx_desc[i].flag = DB_LP;
		rx_desc[i].addr = (virt_to_phys(rx_buff+RXBUFFSIZE*i));;
		//rx_desc[i].addr = (virt_to_phys(NetRxPackets[i]));;
		//printf("rx_desc[i].addr:%x\n",rx_desc[i].addr);
	}
	rx_desc[i].size = 0;
	rx_desc[i].flag = 0;
	rx_desc[i].addr = 0;

	for(i=0; i<NUM_TX_DESC; i++){
		tx_desc[i].size = 1600;
		tx_desc[i].flag = LAST | DB_LP;
		tx_desc[i].addr = 0;
	}
	tx_desc[i].size = 0;
	tx_desc[i].flag = 0;
	tx_desc[i].addr = 0; 

	next_rx = now_rx = 0;
	now_tx = 0;
	
	*rx_dpr = (virt_to_phys(&rx_desc[0]));
	*tx_dpr = (virt_to_phys(&tx_desc[0]));
}


/**************************************************************************/
/* HALT - Turn off ethernet interface                                     */
/**************************************************************************/
static void vr4155_enet_halt(struct eth_device* dev){
	struct vr4155_enet_hw* hw = (struct vr4155_enet_hw*)dev->priv;
	volatile u32 *macc1 = (volatile u32*)(hw->hw_addr+MACC1);
	/* MAC reset TX,RX disable */
	*macc1 &= 0x80000010;
}

static int register_vr4155_enet( void *hw_addr, uint32_t phy_addr, unsigned char*mac,
			     const char *post )
{
	struct eth_device* dev;
	struct vr4155_enet_hw* hw;
	int i;

	dev = (struct eth_device*) malloc(sizeof(*dev));
	memset(dev, 0, sizeof(*dev));
	hw = (struct vr4155_enet_hw*) malloc(sizeof(*hw));
	memset(hw, 0, sizeof(*hw));

	sprintf(dev->name, DEVNAME"%s", post);
	dev->iobase = 0;
	dev->priv   = hw;
	dev->init   = vr4155_enet_init;
	dev->halt   = vr4155_enet_halt;
	dev->send   = vr4155_enet_send;
	dev->recv   = vr4155_enet_recv;
	for (i = 0; i < 6; ++i) {
		dev->enetaddr[i] = mac[i];
	}

	hw->hw_addr = hw_addr;
	hw->phy_addr = phy_addr;

	ether_hw_reset(hw);
	mdio_write(hw, MII_CONTROL, MIICNTL_RESET);
	mdio_write(hw, MII_CONTROL, MIICNTL_AUTO);  /* ANEGEN */
	mdio_write(hw, MII_CONTROL,  MIICNTL_AUTO | MIICNTL_RST_AUTO);  /* restart auto nego */

	eth_register(dev);

#if defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII)
	miiphy_register(dev->name, vr4155_miiphy_read, vr4155_miiphy_write);
#endif
	return 1;
}

int vr4155_enet_initialize(bd_t *bis){
#if defined(CONFIG_HAS_ETH1) && CONFIG_VR4155_ETH == 2
	register_vr4155_enet( (void*)ETH0_BASE, ETH0_PHY_ADDR, bis->bi_enetaddr, "0" );
	register_vr4155_enet( (void*)ETH1_BASE, ETH1_PHY_ADDR, bis->bi_enet1addr, "1" );
#else
	register_vr4155_enet( (void*)ETH0_BASE, ETH0_PHY_ADDR, bis->bi_enetaddr, "" );
#endif
	return 1;
}

#endif /* CONFIG_VR4155_ETH */
